@@ -39,21 +39,25 @@ class TestPySodium(unittest.TestCase):
         pysodium.crypto_stream(32)
 
     def test_crypto_stream_xor(self):
-        pysodium.crypto_stream_xor(b'howdy', len(b'howdy'))
-        pysodium.crypto_stream_xor(b'howdy' * 16, len(b'howdy') * 16)
+        nonce = b'\x00' * pysodium.crypto_stream_NONCEBYTES
+        key = b'\x00' * pysodium.crypto_stream_KEYBYTES
+        pysodium.crypto_stream_xor(b'howdy', len(b'howdy'), nonce, key)
+        pysodium.crypto_stream_xor(b'howdy' * 16, len(b'howdy') * 16, nonce, key)
 
     def test_crypto_generichash(self):
-        pysodium.crypto_generichash(b'howdy')
+        r=pysodium.crypto_generichash(b'howdy')
         pysodium.crypto_generichash(b'howdy', outlen=4)
-        pysodium.crypto_generichash(b'howdy', outlen=6)
+        r6=pysodium.crypto_generichash(b'howdy', outlen=6)
         pysodium.crypto_generichash(b'howdy', outlen=8)
         state = pysodium.crypto_generichash_init()
         pysodium.crypto_generichash_update(state, b'howdy')
-        pysodium.crypto_generichash_final(state)
+        r1=pysodium.crypto_generichash_final(state)
 
         state = pysodium.crypto_generichash_init(outlen=6)
         pysodium.crypto_generichash_update(state, b'howdy')
-        pysodium.crypto_generichash_final(state, outlen=6)
+        r61=pysodium.crypto_generichash_final(state, outlen=6)
+        self.assertEqual(r, r1)
+        self.assertEqual(r6, r61)
 
     def test_crypto_box_pk_from_sk(self):
         pk1, sk = pysodium.crypto_box_keypair()
@@ -87,8 +91,11 @@ class TestPySodium(unittest.TestCase):
     def test_crypto_box_open_detached(self):
         pk, sk = pysodium.crypto_box_keypair()
         n = pysodium.randombytes(pysodium.crypto_box_NONCEBYTES)
-        c, mac = pysodium.crypto_box_detached("howdy", n, pk, sk)
-        pysodium.crypto_box_open_detached(c, mac, n, pk, sk)
+        c, mac = pysodium.crypto_box_detached(b"howdy", n, pk, sk)
+        r = pysodium.crypto_box_open_detached(c, mac, n, pk, sk)
+        self.assertEqual(r, b"howdy")
+        changed = b"\0"*len(c)
+        self.assertRaises(ValueError, pysodium.crypto_box_open_detached,changed, mac, n, pk, sk)
 
     def test_crypto_secretbox_open(self):
         k = pysodium.randombytes(pysodium.crypto_secretbox_KEYBYTES)
@@ -96,7 +103,164 @@ class TestPySodium(unittest.TestCase):
         c = pysodium.crypto_secretbox(b"howdy", n, k)
         pysodium.crypto_secretbox_open(c, n, k)
 
-    def test_crypto_scalarmult_curve25519_base(self):
+    def test_crypto_secretstream_xchacha20poly1305_keygen(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        self.assertEqual(len(key), 32)
+
+    # The following 3 tests verify that no exceptions are raised. Cannot check these
+    # in any more detail as doing so would require internal knowledge of the state and
+    # header structures, which may change. This can be assumed to be correct as long
+    # as 'pull' test passes, it's decrypted values matches the original plain text.
+    def test_crypto_secretstream_xchacha20poly1305_init_push(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+    def test_crypto_secretstream_xchacha20poly1305_init_pull(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+
+    def test_crypto_secretstream_xchacha20poly1305_push(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, 0)
+    #----
+
+    def test_crypto_secretstream_xchacha20poly1305_pull(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        msg, tag = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext, None)
+
+        self.assertEqual(msg, b"howdy")
+        self.assertEqual(tag, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+    def test_crypto_secretstream_xchacha20poly1305_pull_changed_ad(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", b"some data", pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext, b"different data")
+
+
+    def test_crypto_secretstream_xchacha20poly1305_pull_incorrect_key(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        bad_key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, bad_key)
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext, None)
+
+    def test_crypto_secretstream_xchacha20poly1305_pull_multiple(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"Correct Horse Battery Staple", None, 0)
+        ciphertext2 = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        # Verify decryption
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        msg, tag = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext, None)
+        msg2, tag2 = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext2, None)
+
+        self.assertEqual(msg, b"Correct Horse Battery Staple")
+        self.assertEqual(tag, 0)
+
+        self.assertEqual(msg2, b"howdy")
+        self.assertEqual(tag2, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+    def test_crypto_secretstream_xchacha20poly1305_pull_corrupted(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+        ad = 'additional data'
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"Correct Horse Battery Staple", ad, 0)
+
+        # Verify error is raised if cypher text is changed
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext + 'this is a corruption'.encode(), ad)
+
+        # Verify error is raised if additional data is changed
+        ad2 = 'this is not the same'
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext, ad2)
+
+
+    def test_crypto_secretstream_xchacha20poly1305_rekey(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+        # Encrypt two messages with intermediate re-key
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"Correct Horse Battery Staple", None, 0)
+        pysodium.crypto_secretstream_xchacha20poly1305_rekey(state)
+        ciphertext2 = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        # Verify by decrypting them
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        msg, tag = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext, None)
+        pysodium.crypto_secretstream_xchacha20poly1305_rekey(state2)
+        msg2, tag2 = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext2, None)
+
+        self.assertEqual(msg, b"Correct Horse Battery Staple")
+        self.assertEqual(tag, 0)
+
+        self.assertEqual(msg2, b"howdy")
+        self.assertEqual(tag2, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+    def test_crypto_secretstream_xchacha20poly1305_missing_rekey(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+        # Encrypt two messages with intermediate re-key
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"Correct Horse Battery Staple", None, 0)
+        pysodium.crypto_secretstream_xchacha20poly1305_rekey(state)
+        ciphertext2 = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        msg, tag = pysodium.crypto_secretstream_xchacha20poly1305_pull(state2, ciphertext, None)
+        # re-key should be here, so following call should fail
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext2, None)
+
+    def test_crypto_secretstream_xchacha20poly1305_out_of_order_messeges(self):
+        if not pysodium.sodium_version_check(1, 0, 15): return
+
+        key = pysodium.crypto_secretstream_xchacha20poly1305_keygen()
+        state, header = pysodium.crypto_secretstream_xchacha20poly1305_init_push(key)
+
+        ciphertext = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"Correct Horse Battery Staple", None, 0)
+        ciphertext2 = pysodium.crypto_secretstream_xchacha20poly1305_push(state, b"howdy", None, pysodium.crypto_secretstream_xchacha20poly1305_TAG_FINAL)
+
+        # Decrypting the second message first should fail
+        state2 = pysodium.crypto_secretstream_xchacha20poly1305_init_pull(header, key)
+        self.assertRaises(ValueError, pysodium.crypto_secretstream_xchacha20poly1305_pull, state2, ciphertext2, None)
+
+    def test_test_crypto_scalarmult_curve25519_base(self):
         s = pysodium.crypto_scalarmult_curve25519_base(pysodium.randombytes(pysodium.crypto_scalarmult_BYTES))
         r = pysodium.crypto_scalarmult_curve25519_base(pysodium.randombytes(pysodium.crypto_scalarmult_BYTES))
         pysodium.crypto_scalarmult_curve25519(s, r)
@@ -148,12 +312,22 @@ class TestPySodium(unittest.TestCase):
         if not pysodium.sodium_version_check(1, 0, 4): return
         key = binascii.unhexlify(b"4290bcb154173531f314af57f3be3b5006da371ece272afa1b5dbdd1100a1007")
         input_ = binascii.unhexlify(b"86d09974840bded2a5ca")
-        nonce = binascii.unhexlify(b"cd7cf67be39c794a")
+        nonce = binascii.unhexlify(b"cd7cf67be39c794acd7cf67b")
         for ad in [binascii.unhexlify(b"87e229d4500845a079c0"), None]:
             output = pysodium.crypto_aead_chacha20poly1305_ietf_encrypt(input_, ad, nonce, key)
             output = pysodium.crypto_aead_chacha20poly1305_ietf_decrypt(output, ad, nonce, key)
             self.assertEqual(output, input_)
 
+    def test_aead_xchacha20poly1305_ietf(self):
+        if not pysodium.sodium_version_check(1, 0, 12): return
+        key = binascii.unhexlify(b"4290bcb154173531f314af57f3be3b5006da371ece272afa1b5dbdd1100a1007")
+        input_ = binascii.unhexlify(b"86d09974840bded2a5ca")
+        nonce = binascii.unhexlify(b"cd7cf67be39c794acd7cf67bcd7cf67be39c794acd7cf67b")
+        for ad in [binascii.unhexlify(b"87e229d4500845a079c0"), None]:
+            output = pysodium.crypto_aead_xchacha20poly1305_ietf_encrypt(input_, ad, nonce, key)
+            output = pysodium.crypto_aead_xchacha20poly1305_ietf_decrypt(output, ad, nonce, key)
+            self.assertEqual(output, input_)
+
     def test_crypto_stream_chacha20_xor(self):
         key = binascii.unhexlify(b"000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f")
         nonce = binascii.unhexlify(b"0001020304050607")
@@ -179,7 +353,7 @@ class TestPySodium(unittest.TestCase):
         if not pysodium.sodium_version_check(1, 0, 9): return
         pw = "Correct Horse Battery Staple"
         salt = binascii.unhexlify(b'0f58b94c7a369fd8a9a7083e4cd75266')
-        out = pysodium.crypto_pwhash(pysodium.crypto_auth_KEYBYTES, pw, salt, pysodium.crypto_pwhash_OPSLIMIT_INTERACTIVE, pysodium.crypto_pwhash_MEMLIMIT_INTERACTIVE)
+        out = pysodium.crypto_pwhash(pysodium.crypto_auth_KEYBYTES, pw, salt, pysodium.crypto_pwhash_argon2i_OPSLIMIT_INTERACTIVE, pysodium.crypto_pwhash_argon2i_MEMLIMIT_INTERACTIVE, pysodium.crypto_pwhash_ALG_ARGON2I13)
         self.assertEqual(binascii.hexlify(out), b'79db3095517c7358449d84ee3b2f81f0e9907fbd4e0bae4e0bcc6c79821427dc')
 
     def test_crypto_pwhash_storage(self):
@@ -188,12 +362,6 @@ class TestPySodium(unittest.TestCase):
         pstr = pysodium.crypto_pwhash_str(pw, pysodium.crypto_pwhash_OPSLIMIT_INTERACTIVE, pysodium.crypto_pwhash_MEMLIMIT_INTERACTIVE)
         self.assertTrue(pysodium.crypto_pwhash_str_verify(pstr, pw))
 
-    def test_crypto_pwhash_str(self):
-        if not pysodium.sodium_version_check(1, 0, 9): return
-
-    def test_crypto_pwhash_str_verify(self):
-        if not pysodium.sodium_version_check(1, 0, 9): return
-
     def test_crypto_pwhash_scryptsalsa208sha256(self):
         passwd = b'Correct Horse Battery Staple'
         other_passwd = b'correct horse battery staple'
@@ -201,16 +369,16 @@ class TestPySodium(unittest.TestCase):
         other_salt = binascii.unhexlify(b'4206baae5578933d7cfb315b1c257cc7af162965a91a74ccbb1cfa1d747eb692')
 
         # Use very small limits to avoid burning resources in CI
-        mem_limit = 32 * 1024
-        ops_limit = 1024
+        mem_limit = 16777216
+        ops_limit = 32768
 
         key16 = pysodium.crypto_pwhash_scryptsalsa208sha256(16, passwd, salt, ops_limit, mem_limit)
         self.assertEqual(len(key16), 16)
-        self.assertEqual(key16, binascii.unhexlify(b'34f05e9bef8beccd658acf5f123680b7'))
+        self.assertEqual(key16, b'U\x18aL\xcf\xc9\xa3\xf6(\x8f\xed)\xeej8\xdf')
 
         key = pysodium.crypto_pwhash_scryptsalsa208sha256(32, passwd, salt, ops_limit, mem_limit)
         self.assertEqual(len(key), 32)
-        self.assertEqual(key, binascii.unhexlify(b'34f05e9bef8beccd658acf5f123680b7d30c88d7e9328f9e47ab90185b6ee9ff'))
+        self.assertEqual(key, b'U\x18aL\xcf\xc9\xa3\xf6(\x8f\xed)\xeej8\xdfG\x82hf+vu\xcd\x9c\xdb\xbcmt\xf7\xf1\x10')
 
         self.assertNotEqual(key, pysodium.crypto_pwhash_scryptsalsa208sha256(32, passwd, other_salt, ops_limit, mem_limit))
         self.assertNotEqual(key, pysodium.crypto_pwhash_scryptsalsa208sha256(32, other_passwd, salt, ops_limit, mem_limit))
@@ -219,12 +387,11 @@ class TestPySodium(unittest.TestCase):
         passwd = b'Correct Horse Battery Staple'
 
         # Use very small limits to avoid burning resources in CI
-        mem_limit = 32 * 1024
-        ops_limit = 1024
+        mem_limit = 16777216
+        ops_limit = 32768
 
         storage_string = pysodium.crypto_pwhash_scryptsalsa208sha256_str(passwd, ops_limit, mem_limit)
         self.assertTrue(storage_string.startswith(pysodium.crypto_pwhash_scryptsalsa208sha256_STRPREFIX))
-        self.assertNotIn(b'\x00', storage_string)
 
         self.assertNotEqual(storage_string, pysodium.crypto_pwhash_scryptsalsa208sha256_str(passwd, ops_limit, mem_limit), "Each call should compute a new random salt.")
 
@@ -233,8 +400,8 @@ class TestPySodium(unittest.TestCase):
         other_passwd = b'correct horse battery staple'
 
         # Use very small limits to avoid burning resources in CI
-        mem_limit = 32 * 1024
-        ops_limit = 1024
+        mem_limit = 16777216
+        ops_limit = 32768
 
         storage_string = pysodium.crypto_pwhash_scryptsalsa208sha256_str(passwd, ops_limit, mem_limit)
 
@@ -258,7 +425,7 @@ class TestPySodium(unittest.TestCase):
     def test_AsymCrypto_With_Seeded_Keypair(self):
         msg     = b"correct horse battery staple"
         nonce   = pysodium.randombytes(pysodium.crypto_box_NONCEBYTES)
-        pk, sk = pysodium.crypto_box_seed_keypair("howdy")
+        pk, sk = pysodium.crypto_box_seed_keypair(b"\x00" * pysodium.crypto_box_SEEDBYTES)
 
         c = pysodium.crypto_box(msg, nonce, pk, sk)
         m = pysodium.crypto_box_open(c, nonce, pk, sk)
@@ -266,24 +433,28 @@ class TestPySodium(unittest.TestCase):
         self.assertEqual(msg, m)
 
     def test_crypto_hash_sha256(self):
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256("test")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256(b"test")),
             "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256("howdy")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256(b"howdy")),
             "0f1128046248f83dc9b9ab187e16fad0ff596128f1524d05a9a77c4ad932f10a")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256("Correct Horse Battery Staple")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256(b"Correct Horse Battery Staple")),
             "af139fa284364215adfa49c889ab7feddc5e5d1c52512ffb2cfc9baeb67f220e")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256("pysodium")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256(b"pysodium")),
             "0a53ef9bc1bea173118a42bbbe8300abb6bbef83139046940e9593d9559a5df7")
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha256(b"\x80")),
+            "76be8b528d0075f7aae98d6fa57a6d3c83ae480a8469e668d7b0af968995ac71")
 
     def test_crypto_hash_sha512(self):
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512("test")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"test")),
             "ee26b0dd4af7e749aa1a8ee3c10ae9923f618980772e473f8819a5d4940e0db27ac185f8a0e1d5f84f88bc887fd67b143732c304cc5fa9ad8e6f57f50028a8ff")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512("howdy")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"howdy")),
             "905caca5c4685f296c5491d38660d7720ee87bef08f829332e905593522907674de8490de46c969d2c585b40af40439b387562d6f776023507753d1a9554ebbb")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512("Correct Horse Battery Staple")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"Correct Horse Battery Staple")),
             "0675070bda47bef936f0b65ae721d90f82ca137841df4d7cae27776501ae4b446ab926d64dc1d282c8758ac0eb02cc4aa11b2452d4f8ffeb795023b797fe2b80")
-        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512("pysodium")),
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"pysodium")),
             "ecbc6f4ffdb6e6dcbe6e6beecf0b8e05c11b0cc8a56f2b4098cd613585749fcca5ed1cfda3518e33a5d2c63746ee2857ff6857b9a2eeda4cc208c1e7fd89cc17")
+        self.assertEqual(self.byteHashToString(pysodium.crypto_hash_sha512(b"\x80")),
+            "dfe8ef54110b3324d3b889035c95cfb80c92704614bf76f17546ad4f4b08218a630e16da7df34766a975b3bb85b01df9e99a4ec0a1d0ec3de6bed7b7a40b2f10")
 
     def byteHashToString(self, input):
         import sys
